﻿/**
* Sample code for the GetTrackingSummary Canada Post service.
* 
* The GetTrackingSummary service returns the most recent/significant event for a 
* parcel. If it has been delivered, the delivery details are returned. (The parcel 
* is identified using PIN, DNC or the other reference parameters).
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetTrackingSummary
{
    class GetTrackingSummary
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_Tracking_DotNet_Samples\SOAP\tracking\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];

            TrackingPortTypeClient server = new TrackingPortTypeClient("TrackingPort", "https://ct.soa-gw.canadapost.ca/vis/soap/track");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            gettrackingsummaryrequest request = new gettrackingsummaryrequest();
            request.locale = "FR";
            // PIN Request:
            request.ItemElementName = ItemChoiceType1.pin;
            request.Item = "1681334332936901";
            /*  // DNC Request:
             *  request.ItemElementName = ItemChoiceType1.dnc;
             *  request.Item = "315052413796541";
            */
            /*  // Reference Search Request:
             *  request.ItemElementName = ItemChoiceType1.referencecriteria;
             *  referencecriteria refData = new referencecriteria();
             *  refData.destinationpostalcode = "K2H7X3";
             *  refData.referencenumber = "DIA101";
             *  refData.mailingdatefrom = new DateTime(2011, 12, 30);
             *  refData.mailingdateto = new DateTime(2012, 10, 13);
             *  request.Item = refData;
            */

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                gettrackingsummaryresponse response = server.GetTrackingSummary(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(gettrackingsummaryresponseTrackingsummary))
                {
                    // Retrieve values from response object
                    gettrackingsummaryresponseTrackingsummary trackingSummary = (gettrackingsummaryresponseTrackingsummary)response.Item;
                    foreach (var pinSummary in trackingSummary.pinsummary)
                    {
                        responseAsString += "PIN Number: " + pinSummary.pin + "\r\n";
                        responseAsString += "Mailed On Date: " + pinSummary.mailedondate + "\r\n";
                        responseAsString += "Event Description: " + pinSummary.eventdescription + "\r\n\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
